@@ -68,10 +68,11 @@ var updateBackoff = wait.Backoff{
 
 // Controller defines the container runtime config controller.
 type Controller struct {
-	templatesDir string
+	namespace, templatesDir string
 
 	client        mcfgclientset.Interface
 	configClient  configclientset.Interface
+	kubeClient    clientset.Interface
 	eventRecorder record.EventRecorder
 
 	syncHandler                   func(mcp string) error
@@ -106,7 +107,7 @@ type Controller struct {
 
 // New returns a new container runtime config controller
 func New(
-	templatesDir string,
+	namespace, templatesDir string,
 	mcpInformer mcfginformersv1.MachineConfigPoolInformer,
 	ccInformer mcfginformersv1.ControllerConfigInformer,
 	mcrInformer mcfginformersv1.ContainerRuntimeConfigInformer,
@@ -122,9 +123,11 @@ func New(
 	eventBroadcaster.StartRecordingToSink(&coreclientsetv1.EventSinkImpl{Interface: kubeClient.CoreV1().Events("")})
 
 	ctrl := &Controller{
+		namespace:     namespace,
 		templatesDir:  templatesDir,
 		client:        mcfgClient,
 		configClient:  configClient,
+		kubeClient:    kubeClient,
 		eventRecorder: eventBroadcaster.NewRecorder(scheme.Scheme, corev1.EventSource{Component: "machineconfigcontroller-containerruntimeconfigcontroller"}),
 		queue:         workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "machineconfigcontroller-containerruntimeconfigcontroller"),
 		imgQueue:      workqueue.NewRateLimitingQueue(workqueue.DefaultControllerRateLimiter()),
@@ -171,6 +174,10 @@ func New(
 	ctrl.clusterVersionListerSynced = clusterVersionInformer.Informer().HasSynced
 
 	ctrl.patchContainerRuntimeConfigsFunc = ctrl.patchContainerRuntimeConfigs
+	// Add to the queue to trigger a sync when an upgrade happens
+	// this ensures that the capabilities MC is created on an upgrade
+	// This will be removed in the next version
+	ctrl.queue.Add("force-sync-on-upgrade")
 
 	return ctrl
 }
@@ -461,6 +468,17 @@ func (ctrl *Controller) syncContainerRuntimeConfig(key string) error {
 		glog.V(4).Infof("Finished syncing ContainerRuntimeconfig %q (%v)", key, time.Since(startTime))
 	}()
 
+	// First let's create the MC for the default capabilities file
+	// This will be removed in the next version
+	if err := ctrl.createCapabilitiesMC(); err != nil {
+		return fmt.Errorf("failed to create the crio capabilitie MC: %v", err)
+	}
+	// If the key is set to force-sync-on-upgrade, then we can return after creating
+	// the capabilities MC.
+	if key == "force-sync-on-upgrade" {
+		return nil
+	}
+
 	_, name, err := cache.SplitMetaNamespaceKey(key)
 	if err != nil {
 		return err
@@ -504,12 +522,6 @@ func (ctrl *Controller) syncContainerRuntimeConfig(key string) error {
 		return ctrl.syncStatusOnly(cfg, err)
 	}
 
-	if len(mcpPools) == 0 {
-		err := fmt.Errorf("containerRuntimeConfig %v does not match any MachineConfigPools", key)
-		glog.V(2).Infof("%v", err)
-		return ctrl.syncStatusOnly(cfg, err)
-	}
-
 	for _, pool := range mcpPools {
 		role := pool.Name
 		// Get MachineConfig
@@ -522,9 +534,8 @@ func (ctrl *Controller) syncContainerRuntimeConfig(key string) error {
 		// If we have seen this generation and the sync didn't fail, then skip
 		if !isNotFound && cfg.Status.ObservedGeneration >= cfg.Generation && cfg.Status.Conditions[len(cfg.Status.Conditions)-1].Type == mcfgv1.ContainerRuntimeConfigSuccess {
 			// But we still need to compare the generated controller version because during an upgrade we need a new one
-			mcCtrlVersion := mc.Annotations[ctrlcommon.GeneratedByControllerVersionAnnotationKey]
-			if mcCtrlVersion == version.Hash {
-				return nil
+			if mc.Annotations[ctrlcommon.GeneratedByControllerVersionAnnotationKey] == version.Hash {
+				continue
 			}
 		}
 		// Generate the original ContainerRuntimeConfig
@@ -607,6 +618,95 @@ func (ctrl *Controller) mergeConfigChanges(origFile *igntypes.File, cfg *mcfgv1.
 	return cfgTOML, ctrl.syncStatusOnly(cfg, nil)
 }
 
+// createCapabilitiesMC creates a Machine Config for the crio drop-in capabilities file.
+// We are dropping the NET_RAW and SYS_CHROOT capabilitie by default and in order to not break
+// existing clusters, we are creating a drop-in file at /etc/crio/crio.conf.d that has NET_RAW
+// and SYS_CHROOT as part of the default capabilities list. Users then have the option to delete the
+// capabilities MC that creates this drop-in file.
+// Create a crio-capabilities config map to keep track of whether this capabilities MC was already created
+// once. This ensures that if the user deletes the capabilities MC, the controller doesn't recreate it on a
+// resync.
+func (ctrl *Controller) createCapabilitiesMC() error {
+	var configMapName = "crio-capabilities"
+
+	// Check if the crio-capabilities config map exists in the openshift-machine-config-operator namespace
+	capCM, err := ctrl.kubeClient.CoreV1().ConfigMaps(ctrl.namespace).Get(context.TODO(), configMapName, metav1.GetOptions{})
+	capCMIsNotFound := errors.IsNotFound(err)
+	if err != nil && !capCMIsNotFound {
+		return fmt.Errorf("error checking for %s config map: %v", configMapName, err)
+	}
+	// If the crio-capabilities config map exists, that means the capabilities MC was already created
+	// so we should not create this MC again and return
+	if capCM != nil && !capCMIsNotFound {
+		return nil
+	}
+
+	sel, err := metav1.LabelSelectorAsSelector(metav1.AddLabelToSelector(&metav1.LabelSelector{}, builtInLabelKey, ""))
+	if err != nil {
+		return err
+	}
+	// Find all the MachineConfigPools
+	mcpPoolsAll, err := ctrl.mcpLister.List(sel)
+	if err != nil {
+		return err
+	}
+
+	// Create the capabilities MC for all the available pools
+	for _, pool := range mcpPoolsAll {
+		managedKey := getManagedKeyCaps(pool)
+		mc, err := ctrl.client.MachineconfigurationV1().MachineConfigs().Get(context.TODO(), managedKey, metav1.GetOptions{})
+		isNotFound := errors.IsNotFound(err)
+		if err != nil && !isNotFound {
+			return fmt.Errorf("error checking for %s machine config: %v", managedKey, err)
+		}
+		// continue to the next MC if this already exists
+		if mc != nil && !isNotFound {
+			continue
+		}
+
+		tempIgnCfg := ctrlcommon.NewIgnConfig()
+		mc, err = mtmpl.MachineConfigFromIgnConfig(pool.Name, managedKey, tempIgnCfg)
+		if err != nil {
+			return fmt.Errorf("could not create capabilities MachineConfig from new Ignition config: %v", err)
+		}
+		rawCapsIgnition, err := json.Marshal(createNewIgnition(createDefaultCapsFile()))
+		if err != nil {
+			return fmt.Errorf("error marshalling capabilities config ignition: %v", err)
+		}
+		mc.Spec.Config.Raw = rawCapsIgnition
+		// Create the capabilities MC
+		if err := retry.RetryOnConflict(updateBackoff, func() error {
+			_, err = ctrl.client.MachineconfigurationV1().MachineConfigs().Create(context.TODO(), mc, metav1.CreateOptions{})
+			return err
+		}); err != nil {
+			return fmt.Errorf("could not create MachineConfig for capabilities: %v", err)
+		}
+		glog.Infof("Applied Capabilities MC %v on MachineConfigPool %v", managedKey, pool.Name)
+	}
+
+	// Create the config map for capabilities so we know that the capabilities MC has been created
+	capCM.Name = configMapName
+	capCM.Namespace = ctrl.namespace
+	if _, err := ctrl.kubeClient.CoreV1().ConfigMaps(ctrl.namespace).Create(context.TODO(), capCM, metav1.CreateOptions{}); err != nil {
+		return fmt.Errorf("error creating %s config map: %v", configMapName, err)
+	}
+	return nil
+}
+
+// RunCapsBootstrap creates the capabilities mc on bootstrap
+func RunCapsBootstrap(mcpPools []*mcfgv1.MachineConfigPool) ([]*mcfgv1.MachineConfig, error) {
+	var res []*mcfgv1.MachineConfig
+	for _, pool := range mcpPools {
+		capsIgn := createNewIgnition(createDefaultCapsFile())
+		mc, err := mtmpl.MachineConfigFromIgnConfig(pool.Name, getManagedKeyCaps(pool), capsIgn)
+		if err != nil {
+			return nil, err
+		}
+		res = append(res, mc)
+	}
+	return res, nil
+}
+
 func (ctrl *Controller) syncImageConfig(key string) error {
 	startTime := time.Now()
 	glog.V(4).Infof("Started syncing ImageConfig %q (%v)", key, startTime)
