@@ -1,22 +1,27 @@
 package e2e_test
 
 import (
+	"context"
 	"fmt"
 	"strings"
 	"testing"
 	"time"
 
+	ign "github.com/coreos/ignition/config/v2_2"
 	igntypes "github.com/coreos/ignition/config/v2_2/types"
+	ign3types "github.com/coreos/ignition/v2/config/v3_0/types"
 	mcfgv1 "github.com/openshift/machine-config-operator/pkg/apis/machineconfiguration.openshift.io/v1"
 	ctrlcommon "github.com/openshift/machine-config-operator/pkg/controller/common"
 	"github.com/openshift/machine-config-operator/pkg/daemon/constants"
 	"github.com/openshift/machine-config-operator/test/e2e/framework"
+	"github.com/openshift/machine-config-operator/test/helpers"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/fields"
 	"k8s.io/apimachinery/pkg/labels"
+	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/util/uuid"
 	"k8s.io/apimachinery/pkg/util/wait"
 )
@@ -29,13 +34,13 @@ func TestMCDToken(t *testing.T) {
 		LabelSelector: labels.SelectorFromSet(labels.Set{"k8s-app": "machine-config-daemon"}).String(),
 	}
 
-	mcdList, err := cs.Pods("openshift-machine-config-operator").List(listOptions)
+	mcdList, err := cs.Pods("openshift-machine-config-operator").List(context.TODO(), listOptions)
 	require.Nil(t, err)
 
 	for _, pod := range mcdList.Items {
 		res, err := cs.Pods(pod.Namespace).GetLogs(pod.Name, &corev1.PodLogOptions{
 			Container: "machine-config-daemon",
-		}).DoRaw()
+		}).DoRaw(context.TODO())
 		require.Nil(t, err)
 		for _, line := range strings.Split(string(res), "\n") {
 			if strings.Contains(line, "Unable to rotate token") {
@@ -47,7 +52,7 @@ func TestMCDToken(t *testing.T) {
 
 func mcLabelForRole(role string) map[string]string {
 	mcLabels := make(map[string]string)
-	mcLabels["machineconfiguration.openshift.io/role"] = role
+	mcLabels[mcfgv1.MachineConfigRoleLabelKey] = role
 	return mcLabels
 }
 
@@ -66,6 +71,9 @@ func createIgnFile(path, content, fs string, mode int) igntypes.File {
 		Node: igntypes.Node{
 			Filesystem: fs,
 			Path:       path,
+			User: &igntypes.NodeUser{
+				Name: "root",
+			},
 		},
 	}
 }
@@ -76,7 +84,8 @@ func createMCToAddFileForRole(name, role, filename, data, fs string) *mcfgv1.Mac
 	ignConfig := ctrlcommon.NewIgnConfig()
 	ignFile := createIgnFile(filename, "data:,"+data, fs, 420)
 	ignConfig.Storage.Files = append(ignConfig.Storage.Files, ignFile)
-	mcadd.Spec.Config = ignConfig
+	rawIgnConfig := helpers.MarshalOrDie(ignConfig)
+	mcadd.Spec.Config.Raw = rawIgnConfig
 	return mcadd
 }
 
@@ -93,7 +102,7 @@ func TestMCDeployed(t *testing.T) {
 		mcadd := createMCToAddFile("add-a-file", fmt.Sprintf("/etc/mytestconf%d", i), "test", "root")
 
 		// create the dummy MC now
-		_, err := cs.MachineConfigs().Create(mcadd)
+		_, err := cs.MachineConfigs().Create(context.TODO(), mcadd, metav1.CreateOptions{})
 		if err != nil {
 			t.Errorf("failed to create machine config %v", err)
 		}
@@ -120,7 +129,7 @@ func mcdForNode(cs *framework.ClientSet, node *corev1.Node) (*corev1.Pod, error)
 	}
 	listOptions.LabelSelector = labels.SelectorFromSet(labels.Set{"k8s-app": "machine-config-daemon"}).String()
 
-	mcdList, err := cs.Pods("openshift-machine-config-operator").List(listOptions)
+	mcdList, err := cs.Pods("openshift-machine-config-operator").List(context.TODO(), listOptions)
 	if err != nil {
 		return nil, err
 	}
@@ -137,7 +146,7 @@ func TestUpdateSSH(t *testing.T) {
 	cs := framework.NewClientSet("")
 
 	// create a dummy MC with an sshKey for user Core
-	mcName := fmt.Sprintf("sshkeys-worker-%s", uuid.NewUUID())
+	mcName := fmt.Sprintf("00-sshkeys-worker-%s", uuid.NewUUID())
 	mcadd := &mcfgv1.MachineConfig{}
 	mcadd.ObjectMeta = metav1.ObjectMeta{
 		Name:   mcName,
@@ -153,9 +162,10 @@ func TestUpdateSSH(t *testing.T) {
 	}
 	ignConfig := ctrlcommon.NewIgnConfig()
 	ignConfig.Passwd.Users = append(ignConfig.Passwd.Users, tempUser)
-	mcadd.Spec.Config = ignConfig
+	rawIgnConfig := helpers.MarshalOrDie(ignConfig)
+	mcadd.Spec.Config.Raw = rawIgnConfig
 
-	_, err := cs.MachineConfigs().Create(mcadd)
+	_, err := cs.MachineConfigs().Create(context.TODO(), mcadd, metav1.CreateOptions{})
 	require.Nil(t, err, "failed to create MC")
 	t.Logf("Created %s", mcadd.Name)
 
@@ -188,12 +198,14 @@ func TestKernelArguments(t *testing.T) {
 			Labels: mcLabelForWorkers(),
 		},
 		Spec: mcfgv1.MachineConfigSpec{
-			Config:          ctrlcommon.NewIgnConfig(),
+			Config: runtime.RawExtension{
+				Raw: helpers.MarshalOrDie(ctrlcommon.NewIgnConfig()),
+			},
 			KernelArguments: []string{"nosmt", "foo=bar"},
 		},
 	}
 
-	_, err := cs.MachineConfigs().Create(kargsMC)
+	_, err := cs.MachineConfigs().Create(context.TODO(), kargsMC, metav1.CreateOptions{})
 	require.Nil(t, err)
 	t.Logf("Created %s", kargsMC.Name)
 	renderedConfig, err := waitForRenderedConfig(t, cs, "worker", kargsMC.Name)
@@ -219,7 +231,7 @@ func TestKernelArguments(t *testing.T) {
 func TestKernelType(t *testing.T) {
 	cs := framework.NewClientSet("")
 	// Get initial MachineConfig used by the worker pool so that we can rollback to it later on
-	mcp, err := cs.MachineConfigPools().Get("worker", metav1.GetOptions{})
+	mcp, err := cs.MachineConfigPools().Get(context.TODO(), "worker", metav1.GetOptions{})
 	require.Nil(t, err)
 	workerOldMC := mcp.Status.Configuration.Name
 
@@ -229,12 +241,14 @@ func TestKernelType(t *testing.T) {
 			Labels: mcLabelForWorkers(),
 		},
 		Spec: mcfgv1.MachineConfigSpec{
-			Config:     ctrlcommon.NewIgnConfig(),
+			Config: runtime.RawExtension{
+				Raw: helpers.MarshalOrDie(ctrlcommon.NewIgnConfig()),
+			},
 			KernelType: "realtime",
 		},
 	}
 
-	_, err = cs.MachineConfigs().Create(kernelType)
+	_, err = cs.MachineConfigs().Create(context.TODO(), kernelType, metav1.CreateOptions{})
 	require.Nil(t, err)
 	t.Logf("Created %s", kernelType.Name)
 	renderedConfig, err := waitForRenderedConfig(t, cs, "worker", kernelType.Name)
@@ -256,7 +270,7 @@ func TestKernelType(t *testing.T) {
 	}
 
 	// Delete the applied kerneltype MachineConfig to make sure rollback works fine
-	if err := cs.MachineConfigs().Delete(kernelType.Name, &metav1.DeleteOptions{}); err != nil {
+	if err := cs.MachineConfigs().Delete(context.TODO(), kernelType.Name, metav1.DeleteOptions{}); err != nil {
 		t.Error(err)
 	}
 
@@ -285,16 +299,20 @@ func TestKernelType(t *testing.T) {
 func TestPoolDegradedOnFailToRender(t *testing.T) {
 	cs := framework.NewClientSet("")
 
-	mcadd := createMCToAddFile("add-a-file", "/etc/mytestconfs", "test", "")
-	mcadd.Spec.Config.Ignition.Version = "" // invalid, won't render
+	mcadd := createMCToAddFile("add-a-file", "/etc/mytestconfs", "test", "root")
+	ignCfg, _, err := ign.Parse(mcadd.Spec.Config.Raw)
+	require.Nil(t, err, "failed to parse ignition config")
+	ignCfg.Ignition.Version = "" // invalid, won't render
+	rawIgnCfg := helpers.MarshalOrDie(ignCfg)
+	mcadd.Spec.Config.Raw = rawIgnCfg
 
 	// create the dummy MC now
-	_, err := cs.MachineConfigs().Create(mcadd)
+	_, err = cs.MachineConfigs().Create(context.TODO(), mcadd, metav1.CreateOptions{})
 	require.Nil(t, err, "failed to create machine config")
 
 	// verify the pool goes degraded
 	if err := wait.PollImmediate(2*time.Second, 5*time.Minute, func() (bool, error) {
-		mcp, err := cs.MachineConfigPools().Get("worker", metav1.GetOptions{})
+		mcp, err := cs.MachineConfigPools().Get(context.TODO(), "worker", metav1.GetOptions{})
 		if err != nil {
 			return false, err
 		}
@@ -307,13 +325,13 @@ func TestPoolDegradedOnFailToRender(t *testing.T) {
 	}
 
 	// now delete the bad MC and watch pool flipping back to not degraded
-	if err := cs.MachineConfigs().Delete(mcadd.Name, &metav1.DeleteOptions{}); err != nil {
+	if err := cs.MachineConfigs().Delete(context.TODO(), mcadd.Name, metav1.DeleteOptions{}); err != nil {
 		t.Error(err)
 	}
 
 	// wait for the mcp to go back to previous config
 	if err := wait.PollImmediate(2*time.Second, 5*time.Minute, func() (bool, error) {
-		mcp, err := cs.MachineConfigPools().Get("worker", metav1.GetOptions{})
+		mcp, err := cs.MachineConfigPools().Get(context.TODO(), "worker", metav1.GetOptions{})
 		if err != nil {
 			return false, err
 		}
@@ -331,7 +349,9 @@ func TestReconcileAfterBadMC(t *testing.T) {
 
 	// create a MC that contains a valid ignition config but is not reconcilable
 	mcadd := createMCToAddFile("add-a-file", "/etc/mytestconfs", "test", "root")
-	mcadd.Spec.Config.Networkd = igntypes.Networkd{
+	ignCfg, _, err := ign.Parse(mcadd.Spec.Config.Raw)
+	require.Nil(t, err, "failed to parse ignition config")
+	ignCfg.Networkd = igntypes.Networkd{
 		Units: []igntypes.Networkdunit{
 			{
 				Name:     "test.network",
@@ -339,11 +359,13 @@ func TestReconcileAfterBadMC(t *testing.T) {
 			},
 		},
 	}
+	rawIgnCfg := helpers.MarshalOrDie(ignCfg)
+	mcadd.Spec.Config.Raw = rawIgnCfg
 
 	workerOldMc := getMcName(t, cs, "worker")
 
 	// create the dummy MC now
-	_, err := cs.MachineConfigs().Create(mcadd)
+	_, err = cs.MachineConfigs().Create(context.TODO(), mcadd, metav1.CreateOptions{})
 	if err != nil {
 		t.Errorf("failed to create machine config %v", err)
 	}
@@ -373,7 +395,7 @@ func TestReconcileAfterBadMC(t *testing.T) {
 
 	// verify that we got indeed an unavailable machine in the pool
 	if err := wait.Poll(2*time.Second, 5*time.Minute, func() (bool, error) {
-		mcp, err := cs.MachineConfigPools().Get("worker", metav1.GetOptions{})
+		mcp, err := cs.MachineConfigPools().Get(context.TODO(), "worker", metav1.GetOptions{})
 		if err != nil {
 			return false, err
 		}
@@ -386,7 +408,7 @@ func TestReconcileAfterBadMC(t *testing.T) {
 	}
 
 	// now delete the bad MC and watch the nodes reconciling as expected
-	if err := cs.MachineConfigs().Delete(mcadd.Name, &metav1.DeleteOptions{}); err != nil {
+	if err := cs.MachineConfigs().Delete(context.TODO(), mcadd.Name, metav1.DeleteOptions{}); err != nil {
 		t.Error(err)
 	}
 
@@ -401,7 +423,7 @@ func TestReconcileAfterBadMC(t *testing.T) {
 		if err != nil {
 			return false, err
 		}
-		mcp, err := cs.MachineConfigPools().Get("worker", metav1.GetOptions{})
+		mcp, err := cs.MachineConfigPools().Get(context.TODO(), "worker", metav1.GetOptions{})
 		if err != nil {
 			return false, err
 		}
@@ -433,7 +455,7 @@ func TestDontDeleteRPMFiles(t *testing.T) {
 	workerOldMc := getMcName(t, cs, "worker")
 
 	// create the dummy MC now
-	_, err := cs.MachineConfigs().Create(mcHostFile)
+	_, err := cs.MachineConfigs().Create(context.TODO(), mcHostFile, metav1.CreateOptions{})
 	if err != nil {
 		t.Errorf("failed to create machine config %v", err)
 	}
@@ -447,7 +469,7 @@ func TestDontDeleteRPMFiles(t *testing.T) {
 	}
 
 	// now delete the bad MC and watch the nodes reconciling as expected
-	if err := cs.MachineConfigs().Delete(mcHostFile.Name, &metav1.DeleteOptions{}); err != nil {
+	if err := cs.MachineConfigs().Delete(context.TODO(), mcHostFile.Name, metav1.DeleteOptions{}); err != nil {
 		t.Error(err)
 	}
 
@@ -478,7 +500,7 @@ func TestCustomPool(t *testing.T) {
 	createMCP(t, cs, "infra")
 
 	infraMC := createMCToAddFileForRole("infra-host-file", "infra", "/etc/mco-custom-pool", "mco-custom-pool", "root")
-	_, err := cs.MachineConfigs().Create(infraMC)
+	_, err := cs.MachineConfigs().Create(context.TODO(), infraMC, metav1.CreateOptions{})
 	require.Nil(t, err)
 	renderedConfig, err := waitForRenderedConfig(t, cs, "infra", infraMC.Name)
 	require.Nil(t, err)
@@ -497,10 +519,10 @@ func TestCustomPool(t *testing.T) {
 
 	unlabelFunc()
 
-	workerMCP, err := cs.MachineConfigPools().Get("worker", metav1.GetOptions{})
+	workerMCP, err := cs.MachineConfigPools().Get(context.TODO(), "worker", metav1.GetOptions{})
 	require.Nil(t, err)
 	if err := wait.Poll(2*time.Second, 5*time.Minute, func() (bool, error) {
-		node, err := cs.Nodes().Get(infraNode.Name, metav1.GetOptions{})
+		node, err := cs.Nodes().Get(context.TODO(), infraNode.Name, metav1.GetOptions{})
 		require.Nil(t, err)
 		if node.Annotations[constants.DesiredMachineConfigAnnotationKey] != workerMCP.Spec.Configuration.Name {
 			return false, nil
@@ -512,3 +534,55 @@ func TestCustomPool(t *testing.T) {
 	err = waitForPoolComplete(t, cs, "infra", renderedConfig)
 	require.Nil(t, err)
 }
+
+func TestIgn3Cfg(t *testing.T) {
+	cs := framework.NewClientSet("")
+
+	// create a dummy MC with an sshKey for user Core
+	mcName := fmt.Sprintf("99-ign3cfg-worker-%s", uuid.NewUUID())
+	mcadd := &mcfgv1.MachineConfig{}
+	mcadd.ObjectMeta = metav1.ObjectMeta{
+		Name:   mcName,
+		Labels: mcLabelForWorkers(),
+	}
+	// create a new MC that adds a valid user & ssh key
+	testIgn3Config := ign3types.Config{}
+	tempUser := ign3types.PasswdUser{Name: "core", SSHAuthorizedKeys: []ign3types.SSHAuthorizedKey{"1234_test_ign3"}}
+	testIgn3Config.Passwd.Users = append(testIgn3Config.Passwd.Users, tempUser)
+	testIgn3Config.Ignition.Version = "3.0.0"
+	mode := 420
+	testfiledata := "data:,test-ign3-stuff"
+	tempFile := ign3types.File{Node: ign3types.Node{Path: "/etc/testfileconfig"},
+		FileEmbedded1: ign3types.FileEmbedded1{Contents: ign3types.FileContents{Source: &testfiledata}, Mode: &mode}}
+	testIgn3Config.Storage.Files = append(testIgn3Config.Storage.Files, tempFile)
+	rawIgnConfig := helpers.MarshalOrDie(testIgn3Config)
+	mcadd.Spec.Config.Raw = rawIgnConfig
+
+	_, err := cs.MachineConfigs().Create(context.TODO(), mcadd, metav1.CreateOptions{})
+	require.Nil(t, err, "failed to create MC")
+	t.Logf("Created %s", mcadd.Name)
+
+	// grab the latest worker- MC
+	renderedConfig, err := waitForRenderedConfig(t, cs, "worker", mcadd.Name)
+	require.Nil(t, err)
+	err = waitForPoolComplete(t, cs, "worker", renderedConfig)
+	require.Nil(t, err)
+	nodes, err := getNodesByRole(cs, "worker")
+	require.Nil(t, err)
+	for _, node := range nodes {
+		assert.Equal(t, node.Annotations[constants.CurrentMachineConfigAnnotationKey], renderedConfig)
+		assert.Equal(t, node.Annotations[constants.MachineConfigDaemonStateAnnotationKey], constants.MachineConfigDaemonStateDone)
+
+		foundSSH := execCmdOnNode(t, cs, node, "grep", "1234_test_ign3", "/rootfs/home/core/.ssh/authorized_keys")
+		if !strings.Contains(foundSSH, "1234_test_ign3") {
+			t.Fatalf("updated ssh keys not found in authorized_keys, got %s", foundSSH)
+		}
+		t.Logf("Node %s has SSH key", node.Name)
+
+		foundFile := execCmdOnNode(t, cs, node, "cat", "/rootfs/etc/testfileconfig")
+		if !strings.Contains(foundFile, "test-ign3-stuff") {
+			t.Fatalf("updated file doesn't contain expected data, got %s", foundFile)
+		}
+		t.Logf("Node %s has file", node.Name)
+	}
+}
